## R code for Frank et al. 2021 - Exceptional diversity and selection pressure...
## DPP4 analyses

## Determining the unique combination of DPP4 residues

DPP4 <-read.csv("~/Documents/Temp_DPP4/DPP4_contact_residues_2022-01-18.csv")
DPP4full <-DPP4

for(i in 4:18){
  DPP4full<-DPP4full[DPP4full[,i]!="",] # getting rid of ones with blank data
}

#Adding a column for the residues together
DPP4full$seq <-rep("",nrow(DPP4full))
for (i in 4:18){
  for (j in 1:nrow(DPP4full)){
    DPP4full[j,20]<-paste(DPP4full[j,20],DPP4full[j,i],sep="")
  }
  
}
head(DPP4full)

# get the unique types
Unique_types <-as.data.frame(unique(DPP4full$seq),ncol=1)
Unique_types$type <-seq(1:nrow(Unique_types))
colnames(Unique_types)<-c("Seq", "type")
head(Unique_types)
head(DPP4full)
DPP4full$type <-rep("",nrow(DPP4full))

for(i in 1:nrow(DPP4full)){
  Seq <-DPP4full[i,20]
  Type <-Unique_types[Unique_types$Seq==as.character(Seq),2]
  DPP4full[i,21]<-Type
  
}

#write.csv(DPP4full, file="~/Documents/Temp_DPP4/Full_contact_DPP4_seqs_with_type_2022-01-18.csv")
#incomplete sequences were annotated by hand.

#Determining the effect of amino acid changes on the binding residues

## Extracting the results from the aBSREL files ####

require(jsonlite)
JSONS <-list.files(path="~/Documents/Temp_DPP4/", pattern = "*ABSREL.json", full.names=F,recursive=F)
for (j in 1:length(JSONS)){
  Thing <- read_json(paste("~/Documents/Temp_DPP4/",JSONS[j], sep=""), simplifyVector = F)
  Thing[[5]] # <- fifth thing is the number of branches tested and hte number of positive results
  Thing$`branch attributes`$`0`
  #Results <-Thing[[8]]
  OfInterest<-Thing$`branch attributes`$`0`
  ResultTable <-data.frame(matrix(,ncol=4, nrow=length(OfInterest)))
  for(i in 1:length(OfInterest)){
    Temp <-OfInterest[[i]]
    ResultTable[i,1]<-names(OfInterest)[i]
    ResultTable[i,2]<-Temp$LRT
    ResultTable[i,3]<-Temp$`Uncorrected P-value`
    ResultTable[i,4]<-Temp$`Corrected P-value`
  }
  colnames(ResultTable)<-c("Name", "LRT", "Uncorrected p-value", "Corrected p-value")
  ResultTable2 <-ResultTable[1:nrow(ResultTable),]
  write.csv(ResultTable2, file= paste("~/Documents/Temp_DPP4/",strsplit(JSONS[j],".fasta")[[1]][1],"_ABSRELsummary.csv",sep=""))
}

## Looking for the most diverse of the loci for DPP4 ###
Residues <-DPP4
Residues <-Residues[,c(19,1:18)]
str(Residues); Residues$X288
head(Residues); names(Residues)
#Residues[Residues$X28=="FALSE"&!is.na(Residues$X28),10] <-"F"
head(Residues); names(Residues) #5-19

#Making a bat vs. non bat dataset

Chiroptera <-Residues[Residues$Order=="Chiroptera",]
NotBat <-Residues[Residues$Order!="Chiroptera",]

#Load required packages
library(vegan) #version 2.5-7

#All sites
AminoCount <-matrix(nrow=15, ncol=20)
colnames(AminoCount)<-c("A","C","D","E","F","G","H","I","K","L","M","N","P","Q","R","S","T","V","W","Y")
rownames(AminoCount)<-colnames(Residues[5:19])
for(i in 5:19){
  AminoCount[i-4,1]<-nrow(Residues[Residues[,i]=="A",])
  AminoCount[i-4,2]<-nrow(Residues[Residues[,i]=="C",])
  AminoCount[i-4,3]<-nrow(Residues[Residues[,i]=="D",])
  AminoCount[i-4,4]<-nrow(Residues[Residues[,i]=="E",])
  AminoCount[i-4,5]<-nrow(Residues[Residues[,i]=="F",])
  AminoCount[i-4,6]<-nrow(Residues[Residues[,i]=="G",])
  AminoCount[i-4,7]<-nrow(Residues[Residues[,i]=="H",])
  AminoCount[i-4,8]<-nrow(Residues[Residues[,i]=="I",])
  AminoCount[i-4,9]<-nrow(Residues[Residues[,i]=="K",])
  AminoCount[i-4,10]<-nrow(Residues[Residues[,i]=="L",])
  AminoCount[i-4,11]<-nrow(Residues[Residues[,i]=="M",])
  AminoCount[i-4,12]<-nrow(Residues[Residues[,i]=="N",])
  AminoCount[i-4,13]<-nrow(Residues[Residues[,i]=="P",])
  AminoCount[i-4,14]<-nrow(Residues[Residues[,i]=="Q",])
  AminoCount[i-4,15]<-nrow(Residues[Residues[,i]=="R",])
  AminoCount[i-4,16]<-nrow(Residues[Residues[,i]=="S",])
  AminoCount[i-4,17]<-nrow(Residues[Residues[,i]=="T",])
  AminoCount[i-4,18]<-nrow(Residues[Residues[,i]=="V",])
  AminoCount[i-4,19]<-nrow(Residues[Residues[,i]=="W",])
  AminoCount[i-4,20]<-nrow(Residues[Residues[,i]=="Y",])
}


Shannons <-diversity(AminoCount, index="shannon")
Species <-specnumber(AminoCount)
Evenness <-Shannons/log(Species)
cbind(Shannons,Species,Evenness)

# Just diversity in bats 
Thing <-xtabs(~X24,data=Residues)
AminoCountBat <-matrix(nrow=15, ncol=20)
colnames(AminoCountBat)<-c("A","C","D","E","F","G","H","I","K","L","M","N","P","Q","R","S","T","V","W","Y")
rownames(AminoCountBat)<-colnames(Chiroptera[5:19])
for(i in 5:19){
  AminoCountBat[i-4,1]<-nrow(Chiroptera[Chiroptera[,i]=="A",])
  AminoCountBat[i-4,2]<-nrow(Chiroptera[Chiroptera[,i]=="C",])
  AminoCountBat[i-4,3]<-nrow(Chiroptera[Chiroptera[,i]=="D",])
  AminoCountBat[i-4,4]<-nrow(Chiroptera[Chiroptera[,i]=="E",])
  AminoCountBat[i-4,5]<-nrow(Chiroptera[Chiroptera[,i]=="F",])
  AminoCountBat[i-4,6]<-nrow(Chiroptera[Chiroptera[,i]=="G",])
  AminoCountBat[i-4,7]<-nrow(Chiroptera[Chiroptera[,i]=="H",])
  AminoCountBat[i-4,8]<-nrow(Chiroptera[Chiroptera[,i]=="I",])
  AminoCountBat[i-4,9]<-nrow(Chiroptera[Chiroptera[,i]=="K",])
  AminoCountBat[i-4,10]<-nrow(Chiroptera[Chiroptera[,i]=="L",])
  AminoCountBat[i-4,11]<-nrow(Chiroptera[Chiroptera[,i]=="M",])
  AminoCountBat[i-4,12]<-nrow(Chiroptera[Chiroptera[,i]=="N",])
  AminoCountBat[i-4,13]<-nrow(Chiroptera[Chiroptera[,i]=="P",])
  AminoCountBat[i-4,14]<-nrow(Chiroptera[Chiroptera[,i]=="Q",])
  AminoCountBat[i-4,15]<-nrow(Chiroptera[Chiroptera[,i]=="R",])
  AminoCountBat[i-4,16]<-nrow(Chiroptera[Chiroptera[,i]=="S",])
  AminoCountBat[i-4,17]<-nrow(Chiroptera[Chiroptera[,i]=="T",])
  AminoCountBat[i-4,18]<-nrow(Chiroptera[Chiroptera[,i]=="V",])
  AminoCountBat[i-4,19]<-nrow(Chiroptera[Chiroptera[,i]=="W",])
  AminoCountBat[i-4,20]<-nrow(Chiroptera[Chiroptera[,i]=="Y",])
}

ShannonsBat <-diversity(AminoCountBat, index="shannon")
SpeciesBat <-specnumber(AminoCountBat)
EvennessBat<-ShannonsBat/log(SpeciesBat)

cbind(ShannonsBat,SpeciesBat,EvennessBat)

# Diversity in non-bat mammals
AminoCountNotBat <-matrix(nrow=15, ncol=20)
colnames(AminoCountNotBat)<-c("A","C","D","E","F","G","H","I","K","L","M","N","P","Q","R","S","T","V","W","Y")
rownames(AminoCountNotBat)<-colnames(NotBat[5:19])
for(i in 5:19){
  AminoCountNotBat[i-4,1]<-nrow(NotBat[NotBat[,i]=="A",])
  AminoCountNotBat[i-4,2]<-nrow(NotBat[NotBat[,i]=="C",])
  AminoCountNotBat[i-4,3]<-nrow(NotBat[NotBat[,i]=="D",])
  AminoCountNotBat[i-4,4]<-nrow(NotBat[NotBat[,i]=="E",])
  AminoCountNotBat[i-4,5]<-nrow(NotBat[NotBat[,i]=="F",])
  AminoCountNotBat[i-4,6]<-nrow(NotBat[NotBat[,i]=="G",])
  AminoCountNotBat[i-4,7]<-nrow(NotBat[NotBat[,i]=="H",])
  AminoCountNotBat[i-4,8]<-nrow(NotBat[NotBat[,i]=="I",])
  AminoCountNotBat[i-4,9]<-nrow(NotBat[NotBat[,i]=="K",])
  AminoCountNotBat[i-4,10]<-nrow(NotBat[NotBat[,i]=="L",])
  AminoCountNotBat[i-4,11]<-nrow(NotBat[NotBat[,i]=="M",])
  AminoCountNotBat[i-4,12]<-nrow(NotBat[NotBat[,i]=="N",])
  AminoCountNotBat[i-4,13]<-nrow(NotBat[NotBat[,i]=="P",])
  AminoCountNotBat[i-4,14]<-nrow(NotBat[NotBat[,i]=="Q",])
  AminoCountNotBat[i-4,15]<-nrow(NotBat[NotBat[,i]=="R",])
  AminoCountNotBat[i-4,16]<-nrow(NotBat[NotBat[,i]=="S",])
  AminoCountNotBat[i-4,17]<-nrow(NotBat[NotBat[,i]=="T",])
  AminoCountNotBat[i-4,18]<-nrow(NotBat[NotBat[,i]=="V",])
  AminoCountNotBat[i-4,19]<-nrow(NotBat[NotBat[,i]=="W",])
  AminoCountNotBat[i-4,20]<-nrow(NotBat[NotBat[,i]=="Y",])
}

head(AminoCountNotBat)

ShannonsNotBat <-diversity(AminoCountNotBat, index="shannon")
SpeciesNotBat <-specnumber(AminoCountNotBat)
EvennessNotBat<-ShannonsNotBat/log(SpeciesNotBat)

cbind(ShannonsNotBat,SpeciesNotBat,EvennessNotBat)


## Calculating the scores of how similar DPP4 contact residues are to humans
names(Residues)

Residues
Residues$dpp4human<-rep(0,nrow(Residues))
Residues$dpp4bad<-rep(0,nrow(Residues))
Residues$dpp4count<-rep(0,nrow(Residues))

names(Residues)

for (i in 1:nrow(Residues)){
  if(Residues[i,5]%in%c("N")){Residues[i,20]<-(Residues[i,20]+1)}
  if(Residues[i,5]%in%c("D")){Residues[i,21]<-(Residues[i,21]+1)}
  
  if(Residues[i,6]%in%c("K","R")){Residues[i,20]<-(Residues[i,20]+1)}
  
  
  if(Residues[i,7]%in%c("Q")){Residues[i,20]<-(Residues[i,20]+1)}
  if(Residues[i,7]%in%c("P","E")){Residues[i,21]<-(Residues[i,21]+1)}
  
  if(Residues[i,8]%in%c("T","S","N")){Residues[i,20]<-(Residues[i,20]+1)}
  if(Residues[i,8]%in%c("F","P")){Residues[i,21]<-(Residues[i,21]+1)}
  
  if(Residues[i,9]%in%c("L","A","V")){Residues[i,20]<-(Residues[i,20]+1)}
  if(Residues[i,9]%in%c("H","D","E","P","S","T")){Residues[i,21]<-(Residues[i,21]+1)}
  
  if(Residues[i,10]%in%c("L","I","A")){Residues[i,20]<-(Residues[i,20]+1)}
  if(Residues[i,10]%in%c("D","S","E","T")){Residues[i,21]<-(Residues[i,21]+1)}
  
  if(Residues[i,11]%in%c("I","A","V")){Residues[i,20]<-(Residues[i,20]+1)}
  if(Residues[i,11]%in%c("N","K","R","S","T")){Residues[i,21]<-(Residues[i,21]+1)}
  
  if(Residues[i,12]%in%c("H","R")){Residues[i,20]<-(Residues[i,20]+1)}
  if(Residues[i,12]%in%c("Y")){Residues[i,21]<-(Residues[i,21]+1)}
  
  if(Residues[i,13]%in%c("R","K")){Residues[i,20]<-(Residues[i,20]+1)}
  
  if(Residues[i,14]%in%c("Y","F")){Residues[i,20]<-(Residues[i,20]+1)}
  if(Residues[i,14]%in%c("H")){Residues[i,21]<-(Residues[i,21]+1)}
  
  if(Residues[i,15]%in%c("G","A")){Residues[i,20]<-(Residues[i,20]+1)}
  if(Residues[i,15]%in%c("D","E","K","N","P","Q","R","S")){Residues[i,21]<-(Residues[i,21]+1)}
  
  if(Residues[i,16]%in%c("R","H","K")){Residues[i,20]<-(Residues[i,20]+1)}
  if(Residues[i,16]%in%c("E","I","V","L")){Residues[i,21]<-(Residues[i,21]+1)}
  
  if(Residues[i,17]%in%c("L","A","V")){Residues[i,20]<-(Residues[i,20]+1)}
  if(Residues[i,17]%in%c("E","Q","R")){Residues[i,21]<-(Residues[i,21]+1)}
  
  if(Residues[i,18]%in%c("Q")){Residues[i,20]<-(Residues[i,20]+1)}
  if(Residues[i,18]%in%c("E", "K","D","H")){Residues[i,21]<-(Residues[i,21]+1)}
  
  if(Residues[i,19]%in%c("I","L","V","A")){Residues[i,20]<-(Residues[i,20]+1)}
  if(Residues[i,19]%in%c("T","R","D")){Residues[i,21]<-(Residues[i,21]+1)}
  
  
  Count <-Residues[i,5:19]!=""
  Residues[i,22]<-length(Count[Count==T])
}

Residues$dpp4humanscore <-(Residues$dpp4human-Residues$dpp4bad)/Residues$dpp4count
Residues$dpp4humanscore
Residues[Residues$dpp4humanscore<0.3,c(2,19:23)]

Mostcomplete <-Residues[Residues$dpp4count>13,]


## Making the figures ### 

Score_data <-read.csv(file="~/Documents/Temp_DPP4/Scores_for_plotting_2022-01-18.csv")
head(Score_data)
plot(sarsscore~sarsiiscore, data=Score_data)
plot(dpp4score~sarsiiscore, data=Score_data, type="n")

SARSscores <-Score_data[,c(1:3)]
SARS2scores <-Score_data[,c(1,2,4)]
MERS <-Score_data[,c(1,2,5)]

SARSscores$Virus <-rep("SARS-CoV"); names(SARSscores)<-c("Species", "Order", "Score","Virus")
head(SARSscores)
SARS2scores$Virus <-rep("SARS-CoV-2"); names(SARS2scores)<-c("Species", "Order", "Score","Virus")
MERS$Virus <-rep("MERS"); names(MERS)<-c("Species", "Order", "Score","Virus")
Scores <-rbind(SARSscores,SARS2scores, MERS)

Scores[(Scores$Order%in%c("Pholidota","Lagomorpha","Perissodactyla","Macroscelidea","Sirenia","Cingulata","Eulipotyphla","Dermoptera","Pilosa","Scadentia",
                                        "Proboscidea","Diprodontia", "Tubulidentata","Afrosoricida","Monotremata","Didelphimorphia")),2]<-"Other"
Scores[Scores$Order=="Carnivora (Pinnipedia)",2]<-"Carnivora"
Scores[Scores$Order=="Artiodactyla (Cetacea)",2]<-"Artiodactyla"

# this is for plotting Figure S1
Score2 <-Score_data
Score2[(Score2$Order%in%c("Pholidota","Lagomorpha","Perissodactyla","Macroscelidea","Sirenia","Cingulata","Eulipotyphla","Dermoptera","Pilosa","Scadentia",
                          "Proboscidea","Diprodontia", "Tubulidentata","Afrosoricida","Monotremata","Didelphimorphia")),2]<-"Other"
Score2[Score2$Order=="Carnivora (Pinnipedia)",2]<-"Carnivora"
Score2[Score2$Order=="Artiodactyla (Cetacea)",2]<-"Artiodactyla"

## Making figure 2

require(ggplot2)
require(gridExtra)




#getting the susceptibility scores

Thing <-read.csv(file="~/Documents/Temp_DPP4/susceptibility_2022-01-19.csv")
Thing2 <-Thing
Thing<-Thing[Thing$Infection!="",]

#This creates figure 2:
plot1<-ggplot(Scores, aes(x=Order, y=Score, fill=Virus, shape=Virus))+ geom_boxplot(outlier.color=NA) +theme_bw()+labs(y="Similarity to human in residues that contact virus")+scale_fill_manual(values=c("white","gray85","gray55"))+geom_point(position=position_jitterdodge(jitter.width = 0.2, jitter.height = 0.005),size=2, alpha=0.5)+scale_shape_manual(values=c(18,15,16))
graph2 <-ggplot(Thing, aes(x=Infection, y=SARS_2_score))+ geom_boxplot(outlier.color=NA) +theme_bw()+labs(y="Similarity to human in residues that contact SARS-CoV-2")+geom_jitter(aes(shape=Order),width=0.2,height=0.01,size=3, alpha=0.5)
grid.arrange(plot1, graph2, widths=c(2,1),nrow=1)


#This plots figure S1
ggplot(Score2,aes(x=sarsiiscore, y=dpp4score, color=Order, shape=Order))+theme_bw()+geom_jitter(width=0.01, height=0.01,size=3, alpha=0.75)+labs(x="Similarity to human in residues that bind SARS-CoV-2", y="Similarity to human in residues that bind MERS-CoV")




#Checking for variable sites and completeness

AllRes <-read.csv(file="~/Documents/Temp_DPP4/DPP4_character_alignment_2022-01-19.csv")
head(AllRes)
names(AllRes)
length(unique(AllRes$X692)[!is.na(unique(AllRes$X692))]) # gets the number of unique not NA characters
length(unique(AllRes$X82)[!is.na(unique(AllRes$X82))])
length(unique(AllRes$X1))

Test <-unique(AllRes$X715)
Test <-Test[Test!=""]
Test <-Test[!is.na(Test)]


length(Test)

Changing <-matrix(,nrow=ncol(AllRes), ncol=2)
for (i in 5:ncol(AllRes)){
  Changing[i,1] <-colnames(AllRes)[i]
  Test <-unique(AllRes[,i])
  Test <-Test[Test!=""]
  Test <-Test[!is.na(Test)]
  Changing[i,2] <-length(Test)
}
head(Changing)
tail(Changing)
colnames(Changing) <- c("Variable", "Unique_residues")
ChangingDF <-data.frame(Changing)
nrow(ChangingDF[ChangingDF$Unique_residues==1,])
Changing[1:100,]

#write.csv(Changing, file="~/Documents/Temp_DPP4/Variant_sites_DPP4_2022-01-19.csv")


fisher.test(matrix(c(5,10, 73,455),nrow=2))
fisher.test(matrix(c(4,11, 31,497),nrow=2))
fisher.test(matrix(c(4,11, 55,473),nrow=2))
fisher.test(matrix(c(3,12, 38,490),nrow=2))
fisher.test(matrix(c(4,11, 59,469),nrow=2))

fisher.test(matrix(c(12,34,48,449),nrow=2))

head(tAllRes)
tAllRes <-t(AllRes)
tAllResDF <-data.frame(tAllRes)
Sample <-tAllResDF[,8]
Sample2 <-Sample; length(Sample)
Sample2 <-Sample2[!is.na(Sample2)]; length(Sample2)
Sample2 <-Sample2[Sample2!=""]; length(Sample2)
Sample2 <-Sample2[8:length(Sample2)] #770

Sampling <-matrix(,nrow=ncol(tAllRes),ncol=3)
for (i in 1:ncol(tAllRes)){
  Sample <-tAllResDF[,i]
  Sample2 <-Sample; length(Sample)
  Sample2 <-Sample2[!is.na(Sample2)]; length(Sample2)
  Sample2 <-Sample2[Sample2!=""]; length(Sample2)
  Sample3 <-Sample2[8:length(Sample2)] 
  Sampling[i,1]<-as.character(Sample2[1])
  Sampling[i,2]<-as.character(Sample2[2])
  Sampling[i,3]<-length(Sample3)
}

head(Sampling)
#write.csv(Sampling, file="~/Documents/Temp_DPP4/DPP4_coverage_2022-01-30.csv")

#At this point we added in which sequences derived from this study versus publicly available resources
Sampling <-read.csv(file="~/Documents/Temp_DPP4/DPP4_coverage_2022-01-30.csv")
head(Sampling)

shapiro.test(Sampling[Sampling$In_molecular==1,4])

wilcox.test(V3~MyBat, data=Sampling[Sampling$In_molecular==1,], paired=F)
boxplot(V3~MyBat, data=Sampling[Sampling$In_molecular==1,])

mean(Sampling[Sampling$In_molecular==1&Sampling$MyBat==0,4],na.rm=T)
mean(Sampling[Sampling$In_molecular==1&Sampling$MyBat==1,4],na.rm=T)

sd(Sampling[Sampling$In_molecular==1 &Sampling$MyBat==0,4],na.rm=T)/sqrt(length(Sampling[Sampling$In_molecular==1&Sampling$MyBat==0,4]))
sd(Sampling[Sampling$In_molecular==1&Sampling$MyBat==1,4],na.rm=T)/sqrt(length(Sampling[Sampling$In_molecular==1&Sampling$MyBat==1,4]))

## Fisher's exact tests
fisher.test(matrix(c(4,86,5,137),nrow=2,byrow=T))
fisher.test(matrix(c(18,72,16,126),nrow=2,byrow=T))
fisher.test(matrix(c(8,32,16,126),nrow=2,byrow=T))
fisher.test(matrix(c(2,38,5,137),nrow=2,byrow=T))
